#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Description: selinux passwd problem reproducer
#   Author: David Spurek <dspurek@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2012 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/bin/rhts-environment.sh
. /usr/lib/beakerlib/beakerlib.sh

PACKAGE="nss-pam-ldapd"

CONFIG_FILE=""
LDAP_SERVICE=""
LOG_SERVICE=""
BASE64_OPTION=""

PACKAGES=( "pam_ldap"         \
           "nss-pam-ldapd"    \
           "nss-tools"        \
           "expect"           \
           "authconfig"       \
           "openldap"         \
           "openldap-clients" \
           "openldap-servers" \
           "rsyslog"          \
           "openssh-clients"  )
CONFIG_FILE="pam_ldap"
LDAP_SERVICE="slapd"
LOG_SERVICE="rsyslog"
BASE64_OPTION="-d"                    

BDN='cn=Manager,dc=my-domain,dc=com'
PWD='x'

function do_ssh_pass {
expect <<EOF
set timeout 20
spawn ssh ldapuser@my-domain.com
expect "*(yes/no)?*" {send "yes\r"; exp_continue}
expect "ldapuser@my-domain.com's password:" { send "$1\r" }
expect ".*$"
expect "ldapuser@my-domain.com's password:" { exit 3 }
send "whoami\r"
expect "ldapuser" { exit 0 }
    
exit 2
EOF
}

function do_passwd {
expect <<EOF
set timeout 20
spawn passwd ldapuser
expect "password:" {send "$1\r"}
expect "password:" { send "$1\r" }
expect "LDAP password information changed for ldapuser"
expect "passwd: all authentication tokens updated successfully." { exit 0 }
    
exit 2
EOF
}

rlJournalStart

  rlPhaseStartSetup

	for P in ${PACKAGES[@]}; do
		rlCheckRpm $P || rlDie "Package $P is missing"
	done

	rlServiceStop "nscd"
	rlServiceStop "nslcd"
	rlServiceStop $LOG_SERVICE
	rlServiceStop $LDAP_SERVICE

        if [ -e /etc/openldap/cacerts/  ] ; then 
	        rlFileBackup "/etc/openldap/cacerts/" && rlRun "rm -rf /etc/openldap/cacerts/*" 0
        else
                mkdir /etc/openldap/cacerts/
        fi

        if [ -e /etc/$CONFIG_FILE.secret  ] ; then 
	        rlFileBackup "/etc/$CONFIG_FILE.secret"
        fi

	rlFileBackup "/etc/$LOG_SERVICE.conf"    
	rlFileBackup "/etc/sysconfig/iptables"
	rlFileBackup "/etc/$CONFIG_FILE.conf"     
	rlFileBackup "/etc/sysconfig/ldap"      
	rlFileBackup "/etc/openldap/slapd.conf"
	rlFileBackup "/etc/openldap/ldap.conf"      
	rlFileBackup "/etc/openldap/slapd.d" && rm -rf /etc/openldap/slapd.d/*
	rlFileBackup "/var/lib/ldap/" && rm -rf /var/lib/ldap/*
	rlFileBackup "/etc/openldap/slapd.d/" && rlRun "rm -rf /etc/openldap/slapd.d/*" 0
	rlFileBackup "/var/run/openldap/" && rlRun "rm -rf /var/run/openldap/*" 0
	rlFileBackup "/etc/hosts" 
	rlFileBackup "/etc/nsswitch.conf"             

	rlRun "cp -f nsswitch.conf /etc/nsswitch.conf" 0
	rlRun "cp -f ldap /etc/sysconfig/ldap" 0
	rlRun "echo '127.0.0.1 my-domain.com' >> /etc/hosts" 0
	rlRun "cp -f *.pem /etc/openldap/cacerts/" 0
	rlRun "chown ldap:ldap /etc/openldap/cacerts/*" 0
	rlRun "chmod 444 /etc/openldap/cacerts/*" 0      
	rlRun "cacertdir_rehash /etc/openldap/cacerts/" 0-255  
	                
	rlRun "cp -f slapd.conf /etc/openldap/slapd.conf" 0
	slaptest -f /etc/openldap/slapd.conf -F /etc/openldap/slapd.d/
	rlRun "chown -R ldap:ldap /etc/openldap/slapd.d/" 0

	# Loading testing data into OpenLDAP servers.
	rlRun "slapadd -l data.ldif" 0

	rlRun "chown -R ldap:ldap /var/lib/ldap" 0

	        # Enable logging.
	rlRun "echo 'local4.* /var/log/slapd.log' >> /etc/rsyslog.conf" 0
    rlRun "echo '\$SystemLogRateLimitInterval 0' >> /etc/rsyslog.conf" 0

	#Prepare password for rootbinddn option of pam_ldap
	rlRun "echo 'x' > /etc/$CONFIG_FILE.secret" 0
	rlRun "chmod 0600 /etc/$CONFIG_FILE.secret" 0

	rlRun "restorecon -Rv /etc/"    

	rlRun "service $LOG_SERVICE start && sleep 5" 0
	rlRun "service $LDAP_SERVICE start && sleep 10" 0

	#BZ#755975
	# maybe problem with return value 1 of authconfig --savebackup, funcionality ok, but returns 1 instead of 0
	rlRun "authconfig --savebackup backup" 0       
    rlPhaseEnd

    rlPhaseStartTest "Test pam_ldap with no TLS/SSL"     

	#       We need to configure the test system to use pam_ldap for password
	#       checking against a directory server

	rlRun "authconfig --enableldap --enableforcelegacy --disablecache --enableldapauth --updateall \
	  --ldapbasedn dc=my-domain,dc=com --ldapserver ldap://my-domain.com" 0 \
	  "Configuring authentication mechanisms"
	cp /etc/$CONFIG_FILE.conf $CONFIG_FILE.conf	      
	mkdir /home/ldapuser
	echo "" > /var/log/slapd.log 

	rlLog "Change user password using passwd, pam_ldap configured with pam_password clear"
	cp  $CONFIG_FILE.conf /etc/$CONFIG_FILE.conf
	cat /etc/$CONFIG_FILE.conf | grep -v "pam_password" > ${CONFIG_FILE}_passwd.conf
	cp ${CONFIG_FILE}_passwd.conf /etc/$CONFIG_FILE.conf
	rlRun "restorecon -vFR /etc/" 0  
	rlRun "echo 'pam_password clear' >> /etc/$CONFIG_FILE.conf" 0
	rlRun "echo 'binddn uid=binddn,ou=people,dc=my-domain,dc=com' >> /etc/$CONFIG_FILE.conf" 0
	rlRun "echo 'bindpw x' >> /etc/$CONFIG_FILE.conf" 0 
	rlRun "echo 'rootbinddn cn=Manager,dc=my-domain,dc=com' >> /etc/$CONFIG_FILE.conf" 0  
	        
	rlRun "do_passwd test" 0  
	rlRun "do_ssh_pass test" 0  
	echo "" > /var/log/slapd.log


    rlPhaseEnd

    rlPhaseStartTest "Test pam_ldap with SSL" 
   
	rlRun "service $LDAP_SERVICE stop && sleep 5" 

	rlRun "rm -rf /var/lib/ldap/*" 0      
	# Loading testing data into OpenLDAP servers.
	rlRun "slapadd -l data.ldif" 0
	rlRun "chown -R ldap:ldap /var/lib/ldap" 0

	cp  $CONFIG_FILE.conf /etc/$CONFIG_FILE.conf 
	rlRun "restorecon -vFR /etc/" 0

	rlRun "authconfig --restorebackup backup"
	  
	rlRun "service $LDAP_SERVICE start && sleep 5"

	rlRun "authconfig --disablecache --enableldap --enableldapauth --updateall \
	  --ldapbasedn dc=my-domain,dc=com --ldapserver ldaps://my-domain.com" 0 \
	  "Configuring authentication mechanisms" 

		cat /etc/nslcd.conf | grep -v "ssl" > nslcd_nossl.conf
		cp nslcd_nossl.conf /etc/nslcd.conf
		rlRun "restorecon -vFR /etc/" 0
		rlRun "echo 'ssl on' >> /etc/nslcd.conf" 0                   
		rlRun "echo 'binddn cn=Manager,dc=my-domain,dc=com' >> /etc/nslcd.conf" 0
		rlRun "echo 'bindpw x' >> /etc/nslcd.conf" 0
		rlRun "service nslcd restart & sleep 5" 0
	      
	echo "" > /var/log/slapd.log      
	cat /etc/$CONFIG_FILE.conf | grep -v "ssl"  > ${CONFIG_FILE}_nossl.conf
	cp ${CONFIG_FILE}_nossl.conf /etc/$CONFIG_FILE.conf
	rlRun "restorecon -vFR /etc/" 0
	rlRun "echo 'ssl on' >> /etc/$CONFIG_FILE.conf" 0 
             
	rm -f ${CONFIG_FILE}_nossl.conf
	cp  /etc/$CONFIG_FILE.conf ${CONFIG_FILE}_ssl.conf 
	echo "" > /var/log/slapd.log                         

	rlLog "Change user password using passwd, pam_ldap configured with pam_password clear"
	cp  ${CONFIG_FILE}_ssl.conf /etc/$CONFIG_FILE.conf
	cat /etc/$CONFIG_FILE.conf | grep -v "pam_password" > ${CONFIG_FILE}_passwd.conf
	cp ${CONFIG_FILE}_passwd.conf /etc/$CONFIG_FILE.conf
	rlRun "restorecon -vFR /etc/" 0  
	rlRun "echo 'pam_password clear' >> /etc/$CONFIG_FILE.conf" 0
	rlRun "echo 'binddn uid=binddn,ou=people,dc=my-domain,dc=com' >> /etc/$CONFIG_FILE.conf" 0
	rlRun "echo 'bindpw x' >> /etc/$CONFIG_FILE.conf" 0 
	rlRun "echo 'rootbinddn cn=Manager,dc=my-domain,dc=com' >> /etc/$CONFIG_FILE.conf" 0  
	        
	rlRun "do_passwd testssl" 0  
	rlRun "cat /var/log/slapd.log | grep -m 3 ldaps"  
	rlRun "do_ssh_pass testssl" 0   
	echo "" > /var/log/slapd.log

    rlPhaseEnd	  
	  

    rlPhaseStartTest "Test pam_ldap with TLS" 

	rlRun "service $LDAP_SERVICE stop && sleep 5" 

	rlRun "rm -rf /var/lib/ldap/*" 0      
	# Loading testing data into OpenLDAP servers.
	rlRun "slapadd -l data.ldif" 0
	rlRun "chown -R ldap:ldap /var/lib/ldap" 0

	cp  $CONFIG_FILE.conf /etc/$CONFIG_FILE.conf
	rlRun "restorecon -vFR /etc/" 0
  
	rlRun "authconfig --restorebackup backup"
	  
	rlRun "service $LDAP_SERVICE start && sleep 5"

	rlRun "authconfig --disablecache --enableldap --enableldaptls --enableldapauth --updateall \
	  --ldapbasedn dc=my-domain,dc=com --ldapserver ldap://my-domain.com" 0 \
	  "Configuring authentication mechanisms"   

	cat /etc/nslcd.conf | grep -v "ssl" > nslcd_nossl.conf
	cp nslcd_nossl.conf /etc/nslcd.conf
	rlRun "restorecon -vFR /etc/" 0
	rlRun "echo 'ssl start_tls' >> /etc/nslcd.conf" 0   
	rlRun "echo 'binddn cn=Manager,dc=my-domain,dc=com' >> /etc/nslcd.conf" 0
	rlRun "echo 'bindpw x' >> /etc/nslcd.conf" 0
	rlRun "service nslcd restart & sleep 5" 0	

	cp  /etc/$CONFIG_FILE.conf ${CONFIG_FILE}_tls.conf
	rlRun "restorecon -vFR /etc/" 0

	rlLog "Change user password using passwd, pam_ldap configured with pam_password clear"
	cp  ${CONFIG_FILE}_tls.conf /etc/$CONFIG_FILE.conf
	cat /etc/$CONFIG_FILE.conf | grep -v "pam_password" > ${CONFIG_FILE}_passwd.conf
	cp ${CONFIG_FILE}_passwd.conf /etc/$CONFIG_FILE.conf  
	rlRun "restorecon -vFR /etc/" 0
	rlRun "echo 'pam_password clear' >> /etc/$CONFIG_FILE.conf" 0
	rlRun "echo 'binddn uid=binddn,ou=people,dc=my-domain,dc=com' >> /etc/$CONFIG_FILE.conf" 0
	rlRun "echo 'bindpw x' >> /etc/$CONFIG_FILE.conf" 0 
	rlRun "echo 'rootbinddn cn=Manager,dc=my-domain,dc=com' >> /etc/$CONFIG_FILE.conf" 0  
	        
	rlRun "do_passwd test" 0  
	rlRun "cat /var/log/slapd.log | grep 'TLS established'" 
	rlRun "do_ssh_pass test" 0
	echo "" > /var/log/slapd.log                

    rlPhaseEnd

	rlPhaseStartTest "SELinux"
		rlRun "ausearch -m AVC -ts recent >out 2>&1" 0-255
		rlRun "grep '<no matches>' out" 0 "There should not be any SELinux AVC"
		cat out && rm -f out
	rlPhaseEnd
    
  rlPhaseStartCleanup

	rlRun "service $LDAP_SERVICE stop && sleep 10" 0
	rlRun "authconfig --restorebackup backup"
	rlRun "rm -rf /etc/openldap/slapd*" 0 

	# Delete test certificates
	rlRun "rm -f /etc/openldap/cacerts/*.pem" 0

	# Delete test files
	rlRun "rm -rf /var/lib/ldap/*" 0
	rlRun "rm -f /etc/openldap/slapd.conf" 0
	rlRun "rm -rf /var/run/openldap/*" 0   
	rlRun "rm -f /var/log/slapd.log" 0

	rm -r /home/ldapuser
	rm -f $CONFIG_FILE.conf ${CONFIG_FILE}_passwd.conf ${CONFIG_FILE}_tls.conf
	rm -r ${CONFIG_FILE}_ssl.conf

	rlFileRestore

	rlServiceRestore "nscd"
	rlServiceRestore "nslcd"
	rlServiceRestore $LOG_SERVICE
	rlServiceRestore $LDAP_SERVICE

  rlPhaseEnd

rlJournalPrintText

rlJournalEnd

